# -*- coding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2014-TODAY Eynes (<http://www.e-mips.com.ar>)
#
#    This is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import psycopg2
import logging

logger = logging.getLogger(__name__)


def _do_update(cr, installed_version):
    try:
        logger.info('Moving quota_qty & quota_amount from pos_order to account_bank_statement_line')
        cr.execute("""
            -- SQL: Migrate quota_amount & quota_qty from pos.order to account.bank.statement.line
            WITH dataset AS (
                SELECT absl.pos_statement_id, absl.id, po.quota_qty, po.quota_amount
                FROM account_bank_statement_line absl
                    JOIN account_journal aj ON aj.id=absl.journal_id
                    JOIN pos_order po ON po.id=absl.pos_statement_id
                WHERE
                    pos_payment_type ~ 'financial')
            UPDATE account_bank_statement_line absl
            SET quota_amount=ds.quota_amount, quota_qty=ds.quota_qty
            FROM dataset ds WHERE absl.id=ds.id;
        """)
        logger.info('Executing backup to stop using pos_order_return & pos_orders')
        cr.execute('ALTER TABLE pos_order ADD COLUMN is_return_order_2 boolean')
        cr.execute('UPDATE pos_order SET is_return_order_2=is_return_order')
        cr.execute('ALTER TABLE pos_order ADD COLUMN return_order_id_2 numeric')
        cr.execute('UPDATE pos_order SET return_order_id_2=return_order_id')
        cr.execute('ALTER TABLE pos_order ADD COLUMN return_status_2 varchar')
        cr.execute('UPDATE pos_order SET return_status_2=return_status')
    except psycopg2.ProgrammingError as e:
        logger.warning(e)
        cr.rollback()
    except Exception as e:
        logger.warning(e)
        cr.rollback()
    else:
        cr.commit()


def migrate(cr, installed_version):
    return _do_update(cr, installed_version)
