# -*- coding: utf-8 -*-

from openerp import models, fields, api, _
from datetime import datetime
from openerp.tools import DEFAULT_SERVER_DATETIME_FORMAT
from lxml import etree
import logging

_logger = logging.getLogger(__name__)


class account_invoice_line(models.Model):
    _inherit = 'account.invoice.line'

    prodlot_id = fields.Many2one('stock.production.lot')


class account_invoice(models.Model):
    _inherit = 'account.invoice'

    sent_to_ws = fields.Boolean('Sent to WS?')
    request_ids = fields.One2many('ws.request', 'invoice_id', string='WS Requests')
    payments_has_cc = fields.Boolean('Payments has at least one card', compute="compute_payments_has_cc", store=True)

    @api.depends("pos_order_id")
    def compute_payments_has_cc(self):
        for invoice in self:
            if invoice.pos_order_id:
                self.payments_has_cc = any(invoice.pos_order_id.statement_ids.filtered(lambda x: (x.journal_id.pos_payment_type == 'financial')))

    @api.model
    def cron_try_sync_invoices(self):
        self._cr.execute("SELECT id FROM account_invoice WHERE is_massive=False AND sent_to_ws=False AND type IN ('out_invoice', 'out_refund') AND state IN ('open', 'paid');")
        inv_ids = [i[0] for i in self._cr.fetchall()]
        _logger.info('[cron]: Trying massive synchronization with CPZ WS for %s invoices' % len(inv_ids))
        inv_browsed = self.browse(inv_ids)
        for inv in inv_browsed:
            try:
                inv.inform_ws()
            except Exception as e:
                _logger.warning('Cannot send invoice #%s: %s' % (inv.id, e))
        return True

    @api.multi
    def send_data(self, xml):
        self.ensure_one()
        req_id = self.env['ws.request'].create({
            'invoice_id': self.id,
            'xml_req': xml,
        })
        retcode = req_id.send()
        if not retcode:
            self.sent_to_ws = True
            self.message_post(body=_('Invoice informed to credipaz WS.'))
        else:
            self.message_post(body=_('Unable to send invoice to credipaz WS.'))

    @api.multi
    def inform_ws(self):
        for inv in self:
            if not inv.sent_to_ws and inv.type in ('out_invoice', 'out_refund') and inv.state in ('open', 'paid') and not inv.is_massive:
                order = self.env['pos.order'].search([('invoice_id', '=', inv.id)])
                if order:
                    ws_data_order = order.collect_data_for_ws()
                    _logger.debug('Data from pos.order\n%s' % ws_data_order)
                    # ws_data_pick = order.picking_id.collect_data_for_ws()
                    # _logger.debug('Data from stock.picking\n%s' % ws_data_pick)
                    ws_data_inv = inv.collect_data_for_ws()
                    _logger.debug('Data from account.invoice\n%s' % ws_data_inv)
                    # ws_data_order.update(ws_data_pick)
                    ws_data_order.update(ws_data_inv)
                    xml_ready_to_fly = inv.prepare_xml_for_ws(ws_data_order)
                    if xml_ready_to_fly:
                        inv.send_data(xml_ready_to_fly)
                    else:
                        _logger.warning('Omit sending becuse of error in composing XML')
                else:  # No order so not sending anything
                    pass

    @api.model
    def prepare_xml_for_ws(self, data):
        try:
            root = etree.Element('Operacion')
            # Fecha
            fecha_el = etree.SubElement(root, "Fecha")
            fecha_el2 = etree.SubElement(fecha_el, "Fecha")
            fecha_el2.text = data.get('date', '')
            # Hora
            hora_el = etree.SubElement(root, "Hora")
            hora_el2 = etree.SubElement(hora_el, "Hora")
            hora_el2.text = data.get('time', '')
            # Lugar: Sucursal, Terminal, Operador
            lugar_el = etree.SubElement(root, "Lugar")
            sucursal_el = etree.SubElement(lugar_el, "Sucursal")
            terminal_el = etree.SubElement(lugar_el, "Terminal")
            operador_el = etree.SubElement(lugar_el, "Operador")
            sucursal_el.text = data.get('suc_code', '')  # Numero
            terminal_el.text = data.get('terminal_id', '')  # 'PC07SU02'
            operador_el.text = data.get('usuario', '')  # 'CCIRINO'
            # Venta: TipoOperacion, DocumentoComprador, IdVenta, FormaPago, PrecioVenta, DescuentoAplicado, AgenteVentas
            venta_el = etree.SubElement(root, "Venta")
            TipoOperacion_el = etree.SubElement(venta_el, "TipoOperacion")
            TipoOperacion_el.text = data.get('type_operation', '')
            DocumentoComprador_el = etree.SubElement(venta_el, "DocumentoComprador")
            DocumentoComprador_el.text = data.get('customer_document', '00000') or '00000'
            IdVenta_el = etree.SubElement(venta_el, "IdVenta")
            IdVenta_el.text = data.get('voucher_reference', '')
            FormaPago_el = etree.SubElement(venta_el, "FormaPago")
            FormaPago_el.text = data.get('credipaz_payment_code', '')
            PrecioVenta_el = etree.SubElement(venta_el, "PrecioVenta")
            PrecioVenta_el.text = data.get('amount_total', '')
            DescuentoAplicado_el = etree.SubElement(venta_el, "DescuentoAplicado")
            DescuentoAplicado_el.text = data.get('discount_applied', '')
            AgenteVentas_el = etree.SubElement(venta_el, "AgenteVentas")
            AgenteVentas_el.text = data.get('cashier_name', '_')
            # Productos
            productos_el = etree.SubElement(root, "Productos")
            for prod in data.get('products', []):
                producto_el = etree.SubElement(productos_el, "Producto")
                Codigo_el = etree.SubElement(producto_el, "Codigo")
                Codigo_el.text = prod.get('code', '')
                Descripcion_el = etree.SubElement(producto_el, "Descripcion")
                Descripcion_el.text = prod.get('desc', '')
                NumeroSerie_el = etree.SubElement(producto_el, "NumeroSerie")
                NumeroSerie_el.text = prod.get('serial_nr', '')
                PrecioUnitario_el = etree.SubElement(producto_el, "PrecioUnitario")
                PrecioUnitario_el.text = prod.get('price_unit', '')
            # Formas de pago
            formaspago_el = etree.SubElement(root, "FormasDePago")
            for pago in data.get('payments', []):
                fp_el = etree.SubElement(formaspago_el, "FormaDePago")
                TipoPago_el = etree.SubElement(fp_el, "TipoPago")
                TipoPago_el.text = pago.get('type', '')
                ImporteCantidad_el = etree.SubElement(fp_el, "ImporteCantidad")
                ImporteCantidad_el.text = pago.get('amount', '')
                Plazo_el = etree.SubElement(fp_el, "Plazo")
                Plazo_el.text = pago.get('term', '')
                Capital_el = etree.SubElement(fp_el, "Capital")
                Capital_el.text = pago.get('capital', '')
                MarcaTarjeta_el = etree.SubElement(fp_el, "MarcaTarjeta")
                MarcaTarjeta_el.text = pago.get('brand_card', '')
                CuentaTarjeta_el = etree.SubElement(fp_el, "CuentaTarjeta")
                CuentaTarjeta_el.text = pago.get('account_card', '')
                MiembroTarjeta_el = etree.SubElement(fp_el, "MiembroTarjeta")
                MiembroTarjeta_el.text = pago.get('member', '') or ''
                PANTarjeta_el = etree.SubElement(fp_el, "PANTarjeta")
                PANTarjeta_el.text = pago.get('num_card', '')
            # Comprador
            comprador_el = etree.SubElement(root, "Comprador")
            Nombre_el = etree.SubElement(comprador_el, "Nombre")
            Nombre_el.text = data.get('customer_name', '')
            TipoDoc_el = etree.SubElement(comprador_el, "TipoDoc")
            TipoDoc_el.text = data.get('customer_document_type', '')
            Documento_el = etree.SubElement(comprador_el, "Documento")
            Documento_el.text = data.get('customer_document', '00000') or '00000'

            # pretty string
            s = etree.tostring(root, pretty_print=True)
        except Exception as e:
            _logger.warning('Cannot prepare XML due to %s\n%s' % (e, data))
            return False
        else:
            return s

    @api.multi
    def collect_data_for_ws(self):
        self.ensure_one()
        res = {}
        if self.type == 'out_invoice':
            res['voucher_reference'] = str(self.id)
        elif self.type == 'out_refund':  # Search original id that generates this refund
            original_invoice_id = self.pos_order_id.return_order_id.invoice_id.id
            res['voucher_reference'] = str(original_invoice_id)
        else:
            _logger.warning('Cannot determine idVenta for invoice #%s. Using voucher_reference.' % self.id)
            res['voucher_reference'] = self.internal_number.replace('-', '')
        date_dt = datetime.strptime(self.create_date, DEFAULT_SERVER_DATETIME_FORMAT)
        res['date'] = date_dt.strftime('%Y/%m/%d')
        res['time'] = date_dt.strftime('%H:%M')
        res['suc_code'] = self.pos_order_id.session_id.config_id.internal_pos_name  # Numero
        res['terminal_id'] = self.pos_ar_id.shop_id.name  # Nombre
        res['usuario'] = self.user_id.name
        res['type_operation'] = 'VTA' if self.type == 'out_invoice' else 'DEV'
        res['customer_document'] = self.partner_id.vat or ''
        res['customer_document_type'] = self.partner_id.document_type_id.name or '_'
        res['customer_name'] = self.partner_id.name
        res['amount_total'] = str(self.amount_total)
        dto_amount = sum(self.invoice_line.mapped(lambda x: (x.price_unit * (x.discount / float(100))) * x.quantity))
        res['discount_applied'] = str(dto_amount)
        return res

    @api.multi
    def invoice_validate(self):
        res = super(account_invoice, self).invoice_validate()
        try:
            self.inform_ws()
        except Exception as e:
            _logger.warning('Cannot send the invoice to credipaz WS. Will try later.: %s' % e)
        return res


class PosOrder(models.Model):
    _inherit = 'pos.order'

    @api.multi
    def get_invoice_line_values(self, line):
        inv_line = super(PosOrder, self).get_invoice_line_values(line)
        if hasattr(line, 'guarantee_desc') and line.guarantee_desc:
            inv_name = inv_line['name']
            inv_line['name'] = inv_name + ': ' + line.guarantee_desc
        if hasattr(line, 'prodlot_id'):
            inv_line['prodlot_id'] = line.prodlot_id.id
        return inv_line
