# -*- coding: utf-8 -*-
from openerp import models, fields, api, exceptions, _
import logging

_logger = logging.getLogger(__name__)


class ProductPricelist(models.Model):
    _inherit = 'product.pricelist'

    @api.model
    def get_pricelist(self, pos_config_id, credipaz_code):
        if not credipaz_code:
            return None
        params = '-> pos_config_id: %s; credipaz_code: %s' % (pos_config_id, credipaz_code)
        q = """
        WITH q1 AS (
            SELECT ccpr.pricelist_id
            FROM
                pricelist_credipaz_codes pcc
                LEFT JOIN credipaz_codes_pricelist_rel ccpr
                    ON pcc.id=ccpr.code_id
            WHERE
                pcc.name=%(code)s
        ),  -- pricelists matching the code
        q2 AS (
             SELECT pp.id pricelist_id
             FROM product_pricelist pp
                LEFT JOIN pricelist_posconfig_rel ppr
                ON ppr.pricelist_id = pp.id
             WHERE
                pp.type ~ 'sale' AND (
                    (ppr.pricelist_id IS NULL)
                OR
                    (ppr.pos_id=%(pos_config)s)
                )
        ) -- not associated with any pos or exclusively associated with our pos
        SELECT q1.pricelist_id FROM q1 JOIN q2 ON q1.pricelist_id=q2.pricelist_id;
        """
        q_p = {'code': credipaz_code, 'pos_config': pos_config_id}
        self._cr.execute(q, q_p)
        if self._cr.rowcount:
            res, = self._cr.fetchone()
            _logger.info('[get_pricelist]\n%s\n<-%s' % (params, res))
            return res
        else:
            _logger.warning('[get_pricelist]\n%s\n<-%s' % (params, None))
            return None


class PricelistCredipazCodes(models.Model):
    _name = 'pricelist.credipaz.codes'
    _description = 'Credipaz Pricelist Codes (returned by ws)'

    name = fields.Integer(string='Credipaz Code', required=True)
    desc = fields.Char(string='Description', required=True)
    pricelist_ids = fields.Many2many(
        'product.pricelist', 'credipaz_codes_pricelist_rel',
        'code_id', 'pricelist_id', string='Pricelist(s)',
        help='If WS returned this code for a customer and the pricelist is available on pos. That pricelist will be selected.', domain=[('type', '=', 'sale')]
    )

    _sql_constraints = [(
        "check_code_unique",
        "UNIQUE (name)",
        _("Code returned by credipaz WS must be unique")
    )]

    @api.multi
    @api.constrains('pricelist_ids')
    def _check_pricelists(self):
        for item in self:
            if not item.pricelist_ids:
                raise exceptions.ValidationError(_('You must at least select one pricelist'))
