# -*- coding: utf-8 -*-
from openerp import models, fields, api, exceptions, _
from lxml import etree
import logging

_logger = logging.getLogger(__name__)


class stock_picking(models.Model):
    _inherit = 'stock.picking'

    @api.cr_uid_ids_context
    def do_transfer(self, cr, uid, picking_ids, context=None):
        res = super(stock_picking, self).do_transfer(cr, uid, picking_ids, context)
        # Recompute locations of lots involved
        for picking in self.browse(cr, uid, picking_ids, context=context):
            lot_ids = picking.pack_operation_ids.mapped('lot_id')
            lot_ids.action_update_last_location()
        return res


class stock_quant(models.Model):
    _inherit = 'stock.quant'

    @api.model
    def fields_view_get(self, view_id=None, view_type='form', toolbar=False, submenu=False):
        res = super(stock_quant, self).fields_view_get(view_id=view_id, view_type=view_type, toolbar=toolbar, submenu=submenu)
        # spt_id = self._context.get('default_picking_type_id')
        if view_type in ('search', 'tree') and self.env.user.pos_config:
            # spt = self.env['stock.picking.type'].browse(spt_id)
            poscfg = self.env.user.pos_config
            doc = etree.XML(res['arch'])
            locations_pos = [poscfg.stock_location_transit_id.id, poscfg.stock_location_id.id]
            for node in doc.xpath("//filter[@name='my_pos_location']"):
                domain = "[('location_id.id', 'in', %s)]" % locations_pos
                node.set('domain', domain)
            res['arch'] = etree.tostring(doc)
        return res


class stock_production_lot(models.Model):
    _inherit = 'stock.production.lot'

    consignation = fields.Boolean('Consignation')

    @api.model
    def create(self, values):
        if 'product_id' in values:
            consignation = self.env['product.product'].browse(values['product_id']).is_consignation
            if consignation:
                values['consignation'] = consignation
        res = super(stock_production_lot, self).create(values)
        return res

    @api.model
    def search_read(self, domain=None, fields=None, offset=0, limit=None, order=None):
        context = self._context
        cr = self._cr
        if context.get('pos'):
            # Modify domain
            prod_id = context.get('product_id')
            loc_ids = tuple(context.get('loc_ids'))
            cr.execute("""
            SELECT spl.id lot_id FROM stock_production_lot spl
            JOIN stock_quant sq ON sq.lot_id = spl.id
            JOIN stock_location sl ON sl.id = sq.location_id
            LEFT JOIN stock_move sm ON sq.reservation_id = sm.id
            WHERE sq.product_id=%s AND sq.location_id IN %s AND sl.usage ~ 'internal'
            AND (sq.reservation_id IS NULL OR sm.location_dest_id IN %s)  -- Not reserved or Incoming to our locations
            GROUP BY spl.id HAVING sum(sq.qty)>0;
            """, (prod_id, loc_ids, loc_ids))
            lot_ids = [i[0] for i in cr.fetchall()]
            domain = [("id", "in", lot_ids)]
        res = super(stock_production_lot, self).search_read(domain, fields, offset, limit, order)
        return res

    @api.multi
    def check_stock_lot(self):
        """
        Ensure we have enough qty's available in the lot, if not return 0
        """
        self.ensure_one()
        location_ids = tuple([i for i in self._context.get('loc_ids') if i])
        product_id = self._context.get('product_id')
        self._cr.execute("""
        SELECT COALESCE(SUM(sq.qty), 0) qty FROM stock_production_lot spl
        JOIN stock_quant sq ON sq.lot_id = spl.id
        JOIN stock_location sl ON sl.id = sq.location_id
        WHERE sq.product_id=%s AND sl.usage ~ 'internal' AND spl.id=%s;
        """, (product_id, self.id))
        qty = self._cr.fetchone()[0]
        _logger.info("Qty Available %s [product #%s, locations #%s, lot #%s]" % (qty, product_id, location_ids, self.id))
        return 0 if qty <= 0 else qty

    @api.multi
    def read(self, fields=None, load='_classic_read'):
        res = super(stock_production_lot, self).read(fields=fields, load=load)
        for record in self:
            if record.consignation:
                if res[0].get('name', False):
                    res[0]['name'] = res[0]['name'] + _(" (Consignation)")
        return res

    @api.multi
    def name_get(self):
        res = super(stock_production_lot, self).name_get()
        for i, record in enumerate(res):
            production_lot = self.browse(record[0])
            if production_lot.consignation:
                record = (record[0], record[1] + _(" (Consignation)"))
                res[i] = record
        return res

    @api.multi
    def action_update_last_location(self):
        for record in self:
            if record.quant_ids.ids:
                last_quant_id = max(record.quant_ids.ids)
                last_quant_data = self.env['stock.quant'].browse(last_quant_id)
                record.last_location_id = last_quant_data.location_id.id
            else:
                record.last_location_id = False
