# -*- coding: utf-8 -*-
###############################################################################
#   Copyright (c) 2018 Eynes/E-Mips (Santiago Javier Said)
#   License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).
###############################################################################

import logging

from openerp.tools import drop_view_if_exists
from functools import partial


from openerp import models, fields  # , api, fields, _, exceptions
from openerp.addons.decimal_precision import decimal_precision as dp


_logger = logging.getLogger(__name__)


class FinancingReport(models.Model):
    _name = 'financing.report'
    _description = 'Financing Report'
    _auto = False
    _order = 'id'
    _rec_name = 'id'

    def _query(self):
        select_str = """
            SELECT
                row_number() OVER () AS id,
                fc.quota_qty, fc.monthly_rate, fc.coeff, fc.disabled, fc.customer_type,
                fc.journal_id, card_rel.card_id, product_rel.product_id,
                pricelist_rel.pricelist_id, pos_rel.pos_id, fc.id fc_id,
                categ_id.categ_id category_id
            FROM financing_config fc
                LEFT JOIN financing_pos_rel pos_rel
                    ON pos_rel.financing_id=fc.id
                LEFT JOIN financing_card_rel card_rel
                    ON card_rel.financing_id=fc.id
                LEFT JOIN financing_product_rel product_rel
                    ON product_rel.financing_id=fc.id
                LEFT JOIN financing_prod_categ_rel category_rel
                    ON category_rel.financing_id=fc.id
                LEFT JOIN financing_pricelist_rel pricelist_rel
                    ON pricelist_rel.financing_id=fc.id
                LEFT JOIN financing_prod_categ_rel categ_id
                    ON categ_id.financing_id=fc.id
        """
        return select_str

    def init(self, cr):
        if hasattr(super(FinancingReport, self), 'init'):
            super(FinancingReport, self).init(cr)
        drop_view_if_exists(cr, self._table)
        cr.execute("CREATE or REPLACE VIEW %s AS (%s)" % (self._table, self._query()))

    quota_qty = fields.Integer(string='Quota Qty.', readonly=True)
    customer_type = fields.Selection('get_ctype_sel', string='Customer type', readonly=True)
    journal_id = fields.Many2one('account.journal', string='Journal', readonly=True)
    card_id = fields.Many2one('payment.card', string='Card', readonly=True)
    product_id = fields.Many2one('product.product', string='Product', readonly=True)
    pricelist_id = fields.Many2one('product.pricelist', string='Pricelist', readonly=True)
    category_id = fields.Many2one('product.category', string='Category', readonly=True)
    pos_id = fields.Many2one('pos.config', string='PoS Config', readonly=True)
    monthly_rate = fields.Float(string='Monthly Rate', readonly=True, digits=dp.get_precision('Financing Config.'))
    coeff = fields.Float(string='Coeff.', readonly=True, digits=dp.get_precision('Financing Config.'))
    disabled = fields.Boolean(string='Disabled?', readonly=True)
    fc_id = fields.Many2one('financing.config', string='Config', readonly=True)

    def get_ctype_sel(self):
        selection = self.env['financing.config']._fields['customer_type'].selection

        # translate selection labels
        if self.env.lang:
            name = "financing.report,customer_type"
            translate = partial(
                self.env['ir.translation']._get_source, name, 'selection', self.env.lang)
            return [(value, translate(label) if label else label) for value, label in selection]
        else:
            return selection
