/******************************************************************************
 *    Financing - Financing on Odoo
 *    Copyright (C) 2017 Eynes
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Affero General Public License as
 *    published by the Free Software Foundation, either version 3 of the
 *    License, or (at your option) any later version.
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Affero General Public License for more details.
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

function financing_models(instance, module) {

    var _t = instance.web._t;
    var round_pr = instance.web.round_precision;
    var round_di = instance.web.round_decimals;
    var OrderSuper = module.Order;
    module.Order = module.Order.extend({
        displayFinancing: function(product, options){
            // Generate the wizard.
            var fpq_model = new openerp.Model('financing.price.query');
            var customer_type = this.get_client().customer_type;
            var myctx = new instance.web.CompoundContext({'customer_type': customer_type});
            var pricelist_id = this.getPricelist();
            var pos_config_id = this.pos.config.id;
            var fpq_def = fpq_model.call(
                'create', 
                [{
                    product_ids: [[6, 0, [product.id]]],
                    pos_config_id: pos_config_id, 
                    pricelist_id: pricelist_id, 
                    item_ids: []
                }],
                {context: myctx});
            fpq_def.then(function(fpq_model, myctx, res){
                // this is current order
                // Call generate meth
                var fpq_gen_def = fpq_model.call(
                    'generate',
                    [[res], myctx],
                    {},
                    myctx
                );
                fpq_gen_def.then(function(res, data){
                    var fpq_id = res;
                    customer_type = this.get_client().customer_type;
                    var act_window = {
                        'type': 'ir.actions.act_window',
                        'name': 'Financiacion',
                        'res_model': 'financing.price.query.lines',
                        'model': 'financing.price.query.lines',
                        'view_type': 'form',
                        'view_mode': 'list',
                        'views': [[false, 'list']],
                        'action_from': 'financing.price.query.lines',
                        'domain': [['query_id','=', fpq_id]],
                        'target': 'new',
                        'context': {
                            tree_view_ref: 'financing.view_financing_price_query_lines_tree',
                            search_default_customer_type: customer_type,
                            group_by: 'journal_id',
                        },
                        'flags': {
                            sidebar: false,
                            deletable: false,
                            views_switcher: false,
                            action_buttons: false,
                            search_view: true,
                            pager: false,
                            radio: false,
                            // select_view_id: self.parent.fields_view.view_id
                        }
                    };
                    act_manager = new openerp.web.ActionManager();
                    openerp.webclient.action_manager.getChildren()[0].do_action(act_window);
                }.bind(this, res));
            }.bind(this, fpq_model, myctx));
        },
        get_proportional_by_product_id: function(){
            // :returns: {'product_id': rate(0~1), ...}
            var proportionals = {};
            this.get('orderLines').each(function(line){
                var product_id = line.product.id;
                if (!(_.has(proportionals, product_id))) {
                    proportionals[product_id] = line.get_all_prices().priceWithTax; 
                } else {
                    proportionals[product_id] += line.get_all_prices().priceWithTax;
                }
            });
            var TotalTaxIncluded = this.getTotalTaxIncluded();
            _.each(proportionals, function(value,key,obj){obj[key] = (value/TotalTaxIncluded);});
            return proportionals;
        },
        get_financing_data: function(journal_id) {
            var product_ids = _.uniq(_.map(this.get('orderLines').models, function(x){return x.product.id;}));
            var partner_id = this.get_client().id;
            var pricelist_id = this.getCurrentPricelist();
            var pos_config_id = this.pos.config.id;
            var fd = this.pos.get_financing_data(product_ids, journal_id, partner_id, pricelist_id, pos_config_id);
            return fd;
        },
        addProduct: function (product, options) {
            var self = this;
            options = options || {};

            var customer = self.get_client();
            if( !customer ){
                self.pos.pos_widget.screen_selector.show_popup('error',{
                    message: _t('Customer not selected'),
                    comment: _t('Please select a client for this order. This can be done by clicking the order tab')
                });
                return;
            }
            if (product.is_combo && !_.keys(options).includes('from_combo') && !_.keys(options).includes('refund')) {  // Combo is a special case: call custom function only once
               return self.addCombo(product.id); 
            }
            var attr = JSON.parse(JSON.stringify(product));
            attr.pos = this.pos;
            attr.order = this;
            var line = new instance.point_of_sale.Orderline({}, {pos: this.pos, order: this, product: product});
            if (product.track_incoming || product.track_all) {
                if (options.prodlot_id){  // When is a refund, and is lot traceability enabled we should have this
                    var self = this;
                    line.set_serial_id(options.prodlot_id[0]);
                    line.set_serial(options.prodlot_id[1]);
                    sr_no = options.prodlot_id[1];
                    (self.get('orderLines')).each(_.bind( function(item) {
                        if (item.get_product().lot_unique_ok && item.get_product().id == attr.id && item.get_serial() == options.prodlot_id[1]) {
                            alert('Same product is already assigned with same serial number !');
                            sr_no = null;
                            return false;
                        }
                    }, this));
                    if (sr_no != null) {
                        if(options.quantity !== undefined){
                            line.set_quantity(options.quantity);
                        }
                        if(options.price !== undefined){
                            line.set_unit_price(options.price);
                        }
                        if(options.discount !== undefined){
                            line.set_discount(options.discount);
                        }
                        var last_orderline = self.getLastOrderline();
                        if (last_orderline && last_orderline.can_be_merged_with(line) && options.merge !== false){
                            last_orderline.merge(line);
                        } else {
                            self.get('orderLines').add(line);
                        }
                        self.selectLine(self.getLastOrderline());
                    }
                } else {  // Display lot selection wizard
                    var self = this;
                    var sli = self.pos.config.stock_location_id[0];  // stock location
                    var slti = self.pos.config.stock_location_transit_id[0];  // stock transit location
                    var ctx = new instance.web.CompoundContext({'pos': true, 'product_id': attr.id, 'loc_ids': [sli, slti]});
                    // Ask server the price
                    openerp.web.blockUI();
                    var ProductPriceDeferred = this.getActualPrice(product, options);
                    ProductPriceDeferred.then(
                        function(product, options, product_data){
                            options.price = product_data.price;
                            // _super_order.addProduct.call(this, product, options);
                    new instance.web.Model("stock.production.lot").get_func("search_read")
                                (['&','&',['product_id', '=', attr.id],['quant_ids.location_id.usage','=', 'internal'],['quant_ids.location_id.id', 'in', [sli, slti]]], false, false, false, false, ctx).pipe(
                        function(result) {
                            if (result) {
                                initial_ids = _.map(result, function(x) {return x['id']});
				self.pos.serial_numbers = result
                                this.pop = new instance.web.form.SelectCreatePopup(this);
                                    this.pop.select_element(
                                        'stock.production.lot', 
                                        {
                                            title: 'Select serial number for: ' + attr.display_name, 
                                            initial_ids: initial_ids, 
                                            initial_view: 'search',
                                            disable_multiple_selection: true,
                                            no_create: true,
                                        }, [["id", "in", initial_ids]], new instance.web.CompoundContext({}));
                            }
                            this.pop.on("elements_selected", self, function(element_ids) {
                                var dataset = new instance.web.DataSetStatic(self, 'stock.production.lot', {});
                                dataset.name_get(element_ids).done(function(data) {
                                    if (data) {
                                            new instance.web.Model("stock.production.lot").get_func("check_stock_lot")(data[0][0], ctx).pipe(
                                            function(lot_res){
                                                if (lot_res > 0) {
                                                line.set_serial_id(data[0][0]);
                                                    line.set_serial(data[0][1]);
                                                    sr_no = data[0][1];
                                                    (self.get('orderLines')).each(_.bind( function(item) {
                                                        if (item.get_product().lot_unique_ok && item.get_product().id == attr.id && item.get_serial() == data[0][1]) {
                                                            alert('Same product is already assigned with same serial number !');
                                                            sr_no = null;
                                                            return false;
                                                        }
                                                    }, this));
					    	    var current_sn = _.findWhere(self.pos.serial_numbers, {id: data[0][0]})
                                                    if (sr_no != null) {
							line.consignation = current_sn.consignation
                                                        if(options.quantity !== undefined){
                                                            line.set_quantity(options.quantity);
                                                        }
                                                        if(options.price !== undefined){
                                                            line.set_unit_price(options.price);
                                                        }
                                                        if(options.discount !== undefined){
                                                            line.set_discount(options.discount);
                                                        }
                                                        var last_orderline = self.getLastOrderline();
                                                        if (last_orderline && last_orderline.can_be_merged_with(line) && options.merge !== false){
                                                            last_orderline.merge(line);
                                                        } else {
                                                            self.get('orderLines').add(line);
                                                        }
                                                        self.selectLine(self.getLastOrderline());
                                                    }
                                                } else {
                                                    alert (_t('Not enough quantity in this serial number !'));
                                                }
                                                });
                                    }
                                });
                            });
                        });

                            openerp.web.unblockUI();
                        }.bind(this, product, options),
                        function(err, ev){
                            ev.preventDefault();
                            openerp.web.unblockUI();
                            var err_dict = {
                                'message': _t('Price Error'),
                                'comment': _t('Cannot fetch price from server'),
                            };
                            if (err.code === 200) {
                                try {
                                    err_dict.message = err.data.arguments[0];
                                    err_dict.comment = err.data.arguments[1];
                                } catch (e) {
                                    err_dict.comment = err.data.message || err.data.debug;
                                }
                            }
                            window.posmodel.pos_widget.screen_selector.show_popup('error-traceback', err_dict);
                            return;
                        });
                    // Ask server the price
                }
            } else {
                return OrderSuper.prototype.addProduct.call(this, product, options);
            }
        },

        addCombo: function(comboId, options){
            /* Require the lot for each of the product of the combo, add it with discount 100%, add the combo product asking by pricelist the combo price */
            var self = this;
            options = options || {};
            var comboProduct = self.pos.db.get_product_by_id(comboId);
            options.from_combo = 1;
            self.addProduct(comboProduct, options);
            for (var i = 0, len = comboProduct.component_ids.length; i < len; i++) {
                var prod_id = comboProduct.component_ids[i];
                var component = self.pos.db.get_product_by_id(prod_id);
                self.addProduct(component, $.extend(options, {quantity: 1, discount: 100}));
            } 
        },

        getCurrentPricelist: function() {
            var client = this.get_client();
            if (client != null) {
                var pl = client.property_product_pricelist;
                if (pl != null) {
                    return pl[0];
                } else {
                    return this.pos.config.pricelist_id[0];
                }
            } else {  // No customer: default pricelist
                return this.pos.config.pricelist_id[0];
            }
        },
        
        removePaymentline: function(line){
            var res = OrderSuper.prototype.removePaymentline.call(this, line);
            this.pos.pos_widget.payment_screen.update_financing_summary();
            // ^- Refresh UI Summary
            return res;
        },

        addPaymentline: function(cashregister) {
            var self = this;
            if (this.is_return_order === true) {
                var res = OrderSuper.prototype.addPaymentline.call(this, cashregister);
                return res;
            }
            if (cashregister.journal.pos_payment_type == 'financial') {
                var journal_id = cashregister.journal.id;
                var fd = this.pos.get_order().get_financing_data(journal_id);
                fd.then(function(response){
                    if ($.isEmptyObject(response)){
                        try {
                            alertify.error(_t('Unconfigured Financing.'));
                        } catch (e) {
                            console.log('Unconfigured financing');
                        }
                        return false;
                    }
                    var cards_from_server = _.map(_.keys(response), function(el){return parseInt(el);});
                    cards_avail = cashregister.journal.card_ids;
                    cashregister.cards_available = cards_avail;
                    var card_id = cards_avail[0];
                    var quotas = _.map(_.keys(response), function(el){return parseInt(el);});
                    cashregister.quotas_available = quotas;
                    var res = OrderSuper.prototype.addPaymentline.call(this, cashregister);
                    var pl = this.get('paymentLines').at(this.get('paymentLines').length -1);
                    // Store financing data
                    pl.financing_data = response;
                    // Set default card
                    pl.set_card_id(cards_avail[0]);
                    pl.set_quota_qty(quotas[0]);
                    pl.trigger('change:selected', pl);
                    return res;
                }.bind(this), function(err){
                    console.log(err);
                });
            } else {
                var res = OrderSuper.prototype.addPaymentline.call(this, cashregister);
            }
            return res;
        },

        getCashRegisterCredipazIds: function() {  // returns ids of cashregisters with cp card
            var self = this;
            var cashregisters = self.pos.cashregisters;
            var lst = [];
            for (var i = 0, len = cashregisters.length; i < len; i++) {
                if (cashregisters[i].journal.credipaz_card) {
                    lst.push(cashregisters[i].id);
                }
            }
            return lst;
        },
        
        getCashRegisterPointsIds: function() {  // returns ids of cashregisters with points enabled
            var self = this;
            var cashregisters = self.pos.cashregisters;
            var lst = [];
            for (var i = 0, len = cashregisters.length; i < len; i++) {
                if (cashregisters[i].journal.use_points) {
                    lst.push(cashregisters[i].id);
                }
            }
            return lst;
        },

        resolve_partner_data: function(partner_data){
            var self = this;
            var partner = self.pos.db.get_partner_by_id(partner_data['partner_id']);
            partner['ws_ok'] = partner_data['ok'];
            partner['ws_points'] = partner_data['available_points'];
            partner['ws_credit'] = partner_data['available_credit'];
            partner['customer_type'] = partner_data['customer_type'];
        },

        update_partner_ui_tooltip: function (order) {
            client = order.get_client()
            cashregister_cpz_ids = order.getCashRegisterCredipazIds();
            cashregister_pts_ids = order.getCashRegisterPointsIds();
            if (client.ws_ok == false) {  // Disable cashregisters with journal.credipaz_card true
                cashregister_ids = cashregister_cpz_ids.concat(cashregister_pts_ids);
                for (var i = 0, len = cashregister_ids.length; i < len; i++) {
                    cr_id = cashregister_ids[i];
                    var selector = '.paypad-button[cash-register-id="'+cr_id+'"]'
                    cr_ui = posmodel.pos_widget.$(selector);
                    $(cr_ui).attr('disabled', 'disabled');
                }
            }
            if (client.ws_ok == true) {
                for (var i = 0, len = cashregister_pts_ids.length; i < len; i++) {
                    cr_id = cashregister_pts_ids[i];
                    var selector = '.paypad-button[cash-register-id="'+cr_id+'"]'
                    cr_ui = posmodel.pos_widget.$(selector);
                    if (parseFloat(client.ws_points) > 0) {
                        $(cr_ui).removeAttr('disabled');
                    } else {
                        $(cr_ui).attr('disabled', 'disabled');
                    }
                }
                for (var i = 0, len = cashregister_cpz_ids.length; i < len; i++) {
                    cr_id = cashregister_cpz_ids[i];
                    var selector = '.paypad-button[cash-register-id="'+cr_id+'"]'
                    cr_ui = posmodel.pos_widget.$(selector);
                    if (parseFloat(client.ws_credit) > 0) {
                        $(cr_ui).removeAttr('disabled');
                    } else {
                        $(cr_ui).attr('disabled', 'disabled');
                    }
                }
            }
            var tooltip_msg = '<div style="text-align:left;"> Puntos: ' + client.ws_points + '<br />Credito: ' + client.ws_credit + '</div>';
            var order_lst = posmodel.pos_widget.$('.orders .select-order');
            for (var i = 0, len = order_lst.length; i < len; i++) {
                order_ui = order_lst[i];
                order_seq = parseInt($(order_ui).text().trim().match(/([0-9]*)/)[0]);
                if (order_seq == order.sequence_number) {
                    $(order_ui).attr('data-original-title', tooltip_msg);
                    $(order_ui).attr('data-toggle', 'tooltip');
                    $(order_ui).tooltip({delay: {show: 50, hide: 100}});
                } 
            }
        },
        set_client: function(client){
            var self = this;
            openerp.web.blockUI();
            if (client) {
                var cfid = this.pos.config.id;
                var myctx = new instance.web.CompoundContext({config_id: cfid});
                new instance.web.Model('res.partner').call('get_webservice_data',[client.id], {context: myctx}).then(function(partner_data){
                    self.resolve_partner_data(partner_data);
                    openerp.web.unblockUI();
                    // return self.set('client', client);
                    OrderSuper.prototype.set_client.call(this, client);
                    // If pricelist is set in partner_data, try to set it if not keep the default of pos
                    if (partner_data.hasOwnProperty('pricelist_id')) {
                        console.info("The webservice has returned pricelist, using:" + partner_data.pricelist_id);
                        var pl_id = partner_data.pricelist_id;
                        if (_.keys(this.pos.db.pricelist_by_id).includes(String(pl_id))) {
                            this.pos.get_order().set_pricelist(pl_id);   
                            this.pos.pos_widget.pricelist.renderElement();  // <- Refresh UI
                        } else {
                            console.warn("The pricelist is unavailable on this pos.");
                        }
                    }
                    // self.pos.pos_widget.screen_selector.show_popup('error',{
                    //     'message':_t('Warning: Cannot sell to this partner'),
                    //     'comment':_t('Credipaz WS does not allow selling to this customer.'),
                    // });

                }.bind(this),function(err,event){
                    event.preventDefault();
                    openerp.web.unblockUI();
                    var msg = ''
                    if (err.data) {
                        msg = err.data.message;
                    } else {
                        msg = _t('Unknown problem while getting customer data');
                    }
                    self.pos.pos_widget.screen_selector.show_popup('error',{
                        'message':_t('Error: Fetching data from Credipaz WS'),
                        'comment': msg || 'Cannot get data from Credipaz WS, please check your internet connection.',
                    });
                });
            } else {
                // return self.set('client', client);
                OrderSuper.prototype.set_client.call(this, client);
            }
        },

        get_quotas_total: function() {
            var quotas_total = $.map(this.get('orderLines').models, 
                function(line) { return line.quota_amount }).reduce(
                    function(previousValue, currentValue){ return currentValue + previousValue; }, 0
                ); 
            //console.log(quotas_total);
            return quotas_total;
        },

        export_as_JSON: function() {
            res = OrderSuper.prototype.export_as_JSON.call(this);
	    res.consignation = this.consignation;
            res.quota_qty = this.quota_qty;
            res.quota_amount = this.get_quotas_total();
            return res;
        }

    });

    /**
     * Extend the order line
     */
    var OrderlineSuper = module.Orderline;
    module.Orderline = module.Orderline.extend({
        initialize: function(attr, options){
            var self = this;
            res = OrderlineSuper.prototype.initialize.call(this, attr, options);
            self.quota_amount = self.get_product().list_price;
            return res;
        },
        set_discount: function(discount){
            // All products for associated have a maximum discount of 20%
            var disc = Math.min(Math.max(parseFloat(discount) || 0, 0),100);
            var ctype = this.order.get_client().customer_type;
            var allowed_products = [399, 7612, 1326, 7610, 7559, 7561, 7560, 7558, 7557, 7585, 7555, 7553, 7582, 7554, 7556, 7584, 7583, 1297, 394, 1375, 1376, 1377, 7688];
            if ((ctype == 'associated') && (disc != 20.0) && (disc != 0.0) &&
                    (disc != 2.0) && !(_.include(allowed_products, this.product.id)) &&
                    (this.pos.pos_session.config_id[0] != 2) && (this.pos.pos_session.config_id[0] != 32)) {
            // if ((ctype == 'associated') && (disc != 20.0) && (disc != 0.0) && !(_.include(allowed_products, this.product.id))) {
                this.pos.pos_widget.numpad.state.changed.buffer = "0";
                this.pos.pos_widget.numpad.state.attributes.buffer = "0";
                return this.pos.pos_widget.screen_selector.show_popup('error',{
                    'message':_t('Error en descuento'),
                    'comment': _t('El producto solo permite un maximo descuento del 20% para socios.'),
                });
            }
            // if (ctype == 'non_associated') {
            if (ctype == 'non_associated' && (this.pos.pos_session.config_id[0] != 2) && (this.pos.pos_session.config_id[0] != 32)) {
                this.pos.pos_widget.numpad.state.changed.buffer = "0";
                this.pos.pos_widget.numpad.state.attributes.buffer = "0";
                return this.pos.pos_widget.screen_selector.show_popup('error',{
                    'message':_t('Error en descuento'),
                    'comment': _t('El producto no permite descuento para no socios.'),
                });
            }
            res = OrderlineSuper.prototype.set_discount.call(this, discount);
            return res;
        },
    });

    /**
     * Extend the POS model
     */
    var PosModelParent = module.PosModel;
    module.PosModel = module.PosModel.extend({
        /**
         * @param session
         * @param attributes
         */
        initialize: function (session, attributes) {
            var self = this;
            // fectch res.partner's customer_type
            var partner_model = _.find(this.models, function(model){
                return model.model === 'res.partner';
            });
            partner_model.fields.push('customer_type', 'ref');

            // fectch product.product's financing_tables
            var product_model = _.find(this.models, function(model){
                return model.model === 'product.product';
            });
            product_model.fields.push('is_combo');
            product_model.fields.push('component_ids');
            product_model.loaded = function(self, products) { self.db.add_products_with_pos(products, self); };

            // fectch account.journal's use_points, credipaz_card and pos_payment_type fields...
            // XXX: ... and we had to add name because it was not being triggered after we added the 
            // new fields (why?). It seems that when fields is an empty array it reads name field 
            // by default.
            var journal_model = _.find(this.models, function(model){
                return model.model === 'account.journal';
            });
            journal_model.fields.push('name', 'use_points', 'credipaz_card', 'mp_card', 'points_rate', 'pos_payment_type', 'card_ids');

            PosModelParent.prototype.initialize.apply(this, arguments);
            this.financing_engine = new module.FinancingEngine({
                'pos': this,
                'db': this.db,
                'pos_widget': this.pos_widget
            });
            get_fp(self);
            get_cards(self); 
            this.enabled_quotas = [1, 3, 6, 9, 12, 18, 20];
            this.fc_model = new openerp.Model('financing.config');
        },

        do_lookup_retrieve_order: function(models_to_fetch, searchStr){
            var self = this;
            
            var loaded = new $.Deferred();
            var progress = 0;
            var progress_step = 1.0 / models_to_fetch.length;
            var tmp = {}; // this is used to share a temporary state between models loaders
            
            new_models_to_fetch = _.map(models_to_fetch, _.clone);
            // ^-- Awkward cloning, but works!
            // Modify loaded function to only retrieve what is not already present
            for (var i = 0, len = new_models_to_fetch.length; i < len; i++) {
               var model = new_models_to_fetch[i]; 
               if (model.model == 'pos.order') {
                    model.loaded = function(self, orders){ 
                        for (var i = 0, len = orders.length; i < len; i++) {
                        var cobj = orders[i];  
                        if (_.pluck(self.db.pos_all_orders, 'id').indexOf(cobj.id) === -1) {
                            self.db.pos_all_orders.push(cobj); 
                        }
                        }
                    };
                }
               if (model.model == 'pos.order.line') {
                    model.loaded = function(self, wk_order_lines){ 
                        for (var i = 0, len = wk_order_lines.length; i < len; i++) {
                        var cobj = wk_order_lines[i];  
                        if (_.pluck(self.db.pos_all_order_lines, 'id').indexOf(cobj.id) === -1) {
                            self.db.pos_all_order_lines.push(cobj); 
                        }
                        }
                    };
                }
               if (model.model == 'account.bank.statement.line') {
                    model.loaded = function(self, wk_statement_lines){ 
                        for (var i = 0, len = wk_statement_lines.length; i < len; i++) {
                        var cobj = wk_statement_lines[i];  
                        if (_.pluck(self.db.pos_all_statement_lines, 'id').indexOf(cobj.id) === -1) {
                            self.db.pos_all_statement_lines.push(cobj); 
                        } 
                        }
                    };
                }
            }

            function load_model(index){
                if(index >= new_models_to_fetch.length){
                    loaded.resolve();
                }else{
                    var model = new_models_to_fetch[index];
                    var defCtx = typeof model.context === 'function' ? model.context(self,tmp) : model.context;
                    if (typeof(defCtx) === 'undefined') {
                        var ctx = new Object({'searchVal': searchStr});
                    } else {
                        var ctx = {}; 
                        Object.assign(ctx, defCtx);
                        ctx.searchVal = searchStr;
                    }
                    self.pos_widget.loading_message(_t('Loading')+' '+(model.label || model.model || ''), progress);
                    var fields =  typeof model.fields === 'function'  ? model.fields(self,tmp)  : model.fields;
                    var domain =  typeof model.domain === 'function'  ? model.domain(self,tmp)  : model.domain;
                    var context = ctx; 
                    var ids     = typeof model.ids === 'function'     ? model.ids(self,tmp) : model.ids;
                    progress += progress_step;
                    

                    if( model.model ){
                        if (model.ids) {
                            var records = new instance.web.Model(model.model).call('read',[ids,fields],context);
                        } else {
                            var records = new instance.web.Model(model.model).query(fields).filter(domain).context(context).all();
                        }
                        records.then(function(result){
                                try{    // catching exceptions in model.loaded(...)
                                    $.when(model.loaded(self,result,tmp))
                                        .then(function(){ load_model(index + 1); },
                                              function(err){ loaded.reject(err); });
                                }catch(err){
                                    loaded.reject(err);
                                }
                            },function(err){
                                loaded.reject(err);
                            });
                    }else if( model.loaded ){
                        try{    // catching exceptions in model.loaded(...)
                            $.when(model.loaded(self,tmp))
                                .then(  function(){ load_model(index +1); },
                                        function(err){ loaded.reject(err); });
                        }catch(err){
                            loaded.reject(err);
                        }
                    }else{
                        load_model(index + 1);
                    }
                }
            }

            try{
                load_model(0);
            }catch(err){
                loaded.reject(err);
            }

            return loaded;
        },

        get_financing_data: function (product_ids, journal_id, partner_id, pricelist_id, pos_config_id){
            // Returns a deferred
            var self = this;
            // openerp.web.blockUI();
            // this.fc_model.call('get_data', [], {}).then(
            //     function(res){
            //         console.log(res);
            //         openerp.web.unblockUI();
            //     },
            //     function(err){
            //         console.log(err);
            //         openerp.web.unblockUI();
            //     }
            // );
            return this.fc_model.call('get_data', [product_ids, journal_id, partner_id, pricelist_id, pos_config_id], {});
        },
        get_financing_quotas: function (financings) {
            // get the allowed quotas and remove duplicated
            financing_quotas = financings.map(function (f) { 
              return f.quota_qty;
            }).filter(function(elem, index, self) {
                return index == self.indexOf(elem);
            });
            if ( !financing_quotas.includes(1) ) { 
                financing_quotas.push(1);
            }
            financing_quotas.sort(function(a, b) { return a - b; });
            return financing_quotas;
        },

        set_enabled_quotas: function(financings) {
            this.enabled_quotas = this.get_financing_quotas(financings);
        },

        reduce_financing_quotas: function (quotas) {
            var quotas_qty_map = {};
            quotas.map(function (qty) { 
                if (!quotas_qty_map.hasOwnProperty(qty)) { 
                   quotas_qty_map[qty] = 0;
                }
                quotas_qty_map[qty] = quotas_qty_map[qty] + 1;
            });
            return quotas_qty_map
        },

        /**
         * find model based on name
         * @param model_name
         * @returns {{}}
         */
        find_model: function (model_name) {
            var self = this;
            var lookup = {};
            for (var i = 0, len = self.models.length; i < len; i++) {
                if (self.models[i].model === model_name) {
                    lookup[i] = self.models[i]
                }
            }
            return lookup
        },
    });

    // extend to add boolean with cash condition
    var PaymentlineParent = module.Paymentline;
    module.Paymentline = module.Paymentline.extend({
       initialize: function(attributes, options) {
           this.points = 0;
           this.financing_data = null;
           this.quota_qty = null;
           this.quota_amount = null;
           var res = PaymentlineParent.prototype.initialize.apply(this, arguments);
           return res;
       },
        export_as_JSON: function(){
            var res = PaymentlineParent.prototype.export_as_JSON.apply(this);
            res.card_number = this.get_card_number();
            res.card_id = this.get_card_id();
            res.quota_qty = this.get_quota_qty();
            res.quota_amount = this.get_quota_amount();
            return res;
        },
        set_card_number: function(number){
            this.card_number = number;
        },
        do_calculations: function(){  // Called when either card_id, quota_amount, amount changes.
            // This function has to set the quota_amount for this payment_line
            if (this.pos.get_order().is_return_order) {
               console.warn("No calculation performed, is a return order."); 
               return;
            }
            var quota_qty = parseInt(this.quota_qty);
            var card_id = this.card_id;
            var amount = this.amount;
            if ((card_id == null) || (quota_qty == null) || (amount == 0)){
                console.warn('No card_id or quota_qty or amount in zero. Cannot do any calculation');
                return;
            }
            console.info('Doing calculations with quotas_qty '+quota_qty+' card_id '+card_id+' amount '+amount);
            console.info(this.financing_data);
            var order = this.pos.get_order();
            var proportionals = order.get_proportional_by_product_id();
            // Check if current card is present in response
            // if (!(_.has(this.financing_data, card_id))) {  // The current card is not available, raise error and set amount to zero 
            //     this.amount = 0;
            //     this.set_quota_amount(null);
            //     this.pos.pos_widget.screen_selector.show_popup('error',{
            //         'message':_t('Error: Card Not Available'),
            //         'comment': _t('The selected card is not available to allow financial payments. Select another card.'),
            //     });
            //     return;
            // }
            // Check if current quota is available for card selected
            if (!(_.has(this.financing_data, quota_qty))) {  // The current quota is not available, raise error and set amount to zero 
                this.amount = 0;
                this.set_quota_amount(null);
                this.pos.pos_widget.screen_selector.show_popup('error',{
                    'message':_t('Error: Quota Not Available'),
                    'comment': _t('The selected quota is not available for the card selected. Try another quota.'),
                });
                return;
            }
            // Check if all product_ids have a coeff
            var financing_touse = this.financing_data[quota_qty];  // {'prod_id': coeff, ...}
            // var product_ids = _.keys(proportionals);
            // var financing_to_use_pids = _.keys(financing_touse);
            // var allprodcoeff = _.every(product_ids, function(financing_to_use_pids, pid){
            //     return _.include(financing_to_use_pids, pid);
            // }.bind(this, financing_to_use_pids));
            // if (!allprodcoeff) {
            //     this.amount = 0;
            //     this.set_quota_amount(null);
            //     this.pos.pos_widget.screen_selector.show_popup('error',{
            //         'message':_t('Error: Products does not share quota availability.'),
            //         'comment': _t('The selected quota is not available for all the products selected.'),
            //     });
            //     return;
            // } 
            // All checks done, calculate the quota_amount.
            // Prorate coefficients with product proportionals against total
            // var coeff_prorated = _.map(proportionals, function(v,k,o){
            //     return this[k] * v;
            // }, financing_touse);
            // var sumcoeff = _.reduce(coeff_prorated, function(memo, num){return memo + num;}, 0);
            // var quota_amount = ((this.amount * (1 + sumcoeff)) / this.quota_qty).toFixed(2);
            var quota_amount = ((this.amount * financing_touse) / this.quota_qty).toFixed(3);
            this.set_quota_amount(quota_amount);
        },
        refresh_quotas_available: function(card_id) {
            var self = this;
            var quotas = _.map(_.keys(self.financing_data[card_id]), function(el){return parseInt(el);});
            this.cashregister.quotas_available = quotas;
            this.set_quota_qty(quotas[0]);
            this.trigger('change:selected', this);
        },
        set_card_id: function(card_id){  // Refresh the quotas available
            this.card_id = card_id;
            // this.refresh_quotas_available(card_id);
        },
        set_quota_qty: function(quota_qty){
            this.quota_qty = quota_qty;
            this.do_calculations();
        },
        set_quota_amount: function(quota_amount){
            this.quota_amount = quota_amount;
            this.pos.pos_widget.payment_screen.update_financing_summary();
        },
        get_quota_amount: function(){
            return this.quota_amount;
        },
        get_quota_qty: function(){
            return this.quota_qty;
        },
        get_card_id: function(){
            return this.card_id || null;
        },
        get_card_number: function(){
            return this.card_number || "";
        },
        set_amount_and_points: function(value, repaint){
            this.amount = round_di(parseFloat(value) || 0, this.pos.currency.decimals);
            this.points = round_di(parseFloat(value * this.cashregister.journal.points_rate) || 0, this.pos.currency.decimals);
            var client_points = this.pos.get_order().attributes.client.ws_points || 0;
            if (this.points > client_points) {
                return this.pos.pos_widget.screen_selector.show_popup('error',{
                    'message':_t('Error: Not enough points'),
                    'comment': _t('Trying to use more points that customer has available.'),
                });
            }
            this.trigger('change:amount',this);
            repaint && this.trigger('change:selected', this);
        },
        set_points_and_amount: function(value, repaint){
            this.points = round_di(parseFloat(value) || 0, this.pos.currency.decimals);
            var client_points = this.pos.get_order().attributes.client.ws_points || 0;
            if (this.points > client_points) {
                return this.pos.pos_widget.screen_selector.show_popup('error',{
                    'message':_t('Error: Not enough points'),
                    'comment': _t('Trying to use more points that customer has available.'),
                });
            }
            this.amount = round_di(parseFloat(value / this.cashregister.journal.points_rate) || 0, this.pos.currency.decimals);
            this.trigger('change:amount',this);
            repaint && this.trigger('change:selected', this);
        },
        get_points: function(){
            var res = this.amount * this.cashregister.journal.points_rate;
            return res;
        },
        get_points_str: function(){
            return openerp.instances[this.pos.session.name].web.format_value(this.points, {
                type: 'float', digits: [69, this.pos.currency.decimals]
            });
        },
        set_amount: function(value){
         res = PaymentlineParent.prototype.set_amount.call(this, value);
         if (this.cashregister.journal.pos_payment_type === 'financial') {
             this.do_calculations();
         }
         return res
        }
    })

    /**
     * Financing Engine to show the fiancings
     */
    module.FinancingEngine = instance.web.Class.extend({
        /**
         * @param options
         */
        init: function (options) {
            options = options || {};
            this.pos = options.pos;
            this.db = options.db;
            this.pos_widget = options.pos_widget;
        },

        set_product_ui_tooltip: function (product_ui, table) {
            var table_header = '<table class="tooltip_table">'
              + '<tr><th class="tooltip_table_header">Cuotas</th>' 
              + '<th class="tooltip_table_header">$</th>' 
              + '<th class="tooltip_table_header">Total</th>' 
              + '<th class="tooltip_table_header">T.E.A.</th>' 
              + '<th class="tooltip_table_header">C.F.T.</th></tr>';
            var closing_tag = '</table>';
            $(product_ui).find('.product-img').attr(
                'data-original-title', table_header + table + closing_tag
            );
            $(product_ui).find('.product-img').attr('data-toggle', 'tooltip');
            $(product_ui).find('.product-img').tooltip({delay: {show: 50, hide: 100}});
        },

    });

    function get_fp(pos_model) {
        var res_product_pricelist = pos_model.find_model('product.pricelist');
        if (_.size(res_product_pricelist) >= 1) {
            var pricelist_index = parseInt(Object.keys(res_product_pricelist)[0]);
            pos_model.models.splice(++pricelist_index, 0, 
                {
                    model: 'account.fiscal.position',
                    fields: ['name', 'id'],
                    domain: null,
                    loaded: function (self, fiscal_positions) {
                        self.fiscal_positions = fiscal_positions;
                    }
                });
        }
    };


    function get_cards(pos_model) {

        var journal_model = pos_model.find_model('account.journal');
        if (_.size(journal_model) == 1) {
            var journal_index = parseInt(Object.keys(journal_model)[0]);
            pos_model.models.splice(++journal_index, 0, 
                {
                    model: 'payment.card',
                    fields: ['name', 'id'],
                    domain: null,
                    loaded: function (self, payment_cards) {
                        self.payment_cards = payment_cards;
                        var journals = self.journals;
                        for (var i = 0, len = journals.length; i < len; i++) {
                            the_journal = journals[i];
                            the_journal.cards = [];
                            for (var j = 0, len2 = payment_cards.length; j < len2; j++) {
                                if (journals[i].card_ids.includes(payment_cards[j].id)){
                                    the_journal.cards.push(payment_cards[j]);
                                }
                            }
                        }
                    }
                });
        }
    }
}
