/******************************************************************************
*    Point Of Sale - Pricelist for POS Odoo
*    Copyright (C) 2014 Taktik (http://www.taktik.be)
*    @author Adil Houmadi <ah@taktik.be>
*
*    This program is free software: you can redistribute it and/or modify
*    it under the terms of the GNU Affero General Public License as
*    published by the Free Software Foundation, either version 3 of the
*    License, or (at your option) any later version.
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU Affero General Public License for more details.
*    You should have received a copy of the GNU Affero General Public License
*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
******************************************************************************/
function financing_screens(instance, module) {

    var _t = instance.web._t;
    var round_pr = instance.web.round_precision;
    var round_di = instance.web.round_decimals;
            
    module.ProductListWidget = module.ProductListWidget.extend({
        init: function(parent, options){
            var self = this;
            this.options = options || [];
            this.parent = parent || {};
            options.click_financing_action = this.click_financing_action;
            this._super(parent, options);
            this.pos = parent.pos;
            this.click_financing_handler = function(event){
                event.stopPropagation();
                var el = event.target;
                while (true){
                    pid = parseInt(el.getAttribute('data-product-id'));
                    if (isNaN(pid)) {
                        el = el.parentElement;
                    } else {
                        break;
                    }
                }
                var product = self.pos.db.get_product_by_id(pid);
                options.click_financing_action(product);
            };
        },
        renderElement: function() {
            var self = this;

            this._super();
            var financings_elements = $(this.el).find('span.financing-tag');
            for(var i = 0, len = financings_elements.length; i < len; i++){
                var g_node = financings_elements[i];
                g_node.addEventListener('click',this.click_financing_handler);
            }
        },
        click_financing_action: function(product){
            var selectedOrder = window.posmodel.get('selectedOrder');
            selectedOrder.displayFinancing(product);
        },
    });

    module.ClientListScreenWidget = module.ClientListScreenWidget.extend({
        save_client_details: function(partner) {
            var self = this;
            
            var fields = {};
            this.$('.client-details-contents .detail').each(function(idx,el){
                fields[el.name] = el.value;
            });

            if (!fields.name && !fields.ref) {
                this.pos_widget.screen_selector.show_popup('error',{
                    message: _t('A Customer Name or Customer Account Is Required'),
                });
                return;
            }
            
            if (this.uploaded_picture) {
                fields.image = this.uploaded_picture;
            }

            fields.id           = partner.id || false;
            fields.country_id   = fields.country_id || false;
            fields.ean13        = fields.ean13 ? this.pos.barcode_reader.sanitize_ean(fields.ean13) : false; 

            new instance.web.Model('res.partner').call('create_from_ui',[fields]).then(function(partner_id){
                self.saved_client_details(partner_id);
            },function(err,event){
                //console.info(err);
                event.preventDefault();
                if (err.data.message) {
                    comment = _t(err.data.message);
                } else {
                    comment = _t('Your Internet connection is probably down.');
                }
                self.pos_widget.screen_selector.show_popup('error',{
                    'message':_t('Error: Could not Save Changes'),
                    'comment': comment,
                });
            });
        },
    });

    module.PaymentScreenWidget = module.PaymentScreenWidget.extend({
        init: function(parent, options) {
            var self = this;
            this._super(parent,options);
            this.line_card_handler = function(event){
                var node = this;
                while(node && !node.classList.contains('paymentline')){
                    node = node.parentNode;
                }
                if (node) {
                    try{
                        card_number = this.value;
                    }
                    catch(e){
                        card_number = "";
                    } node.line.set_card_number(card_number);
                }
            };
            this.line_mp_input_handler = function(event){
                var node = this;
                while(node && !node.classList.contains('paymentline')){
                    node = node.parentNode;
                }
                if (node) {
                    try{
                        card_id = this.value;
                    }
                    catch(e){
                        card_id = "";
                    }
                    node.line.set_card_id(card_id);
                }
            };
            this.line_mp_qty_handler = function(event){
                var node = this;
                while(node && !node.classList.contains('paymentline')){
                    node = node.parentNode;
                }
                if (node) {
                    try{
                        quota_qty = this.value;
                    }
                    catch(e){
                        quota_qty = 1;
                    }
                    node.line.set_quota_qty(quota_qty);
                }
            };
            this.line_change_handler = function(event){
                var node = this;
                while(node && !node.classList.contains('paymentline')){
                    node = node.parentNode;
                }
                if(node){
                    var amount;
                    try{
                        amount = instance.web.parse_value(this.value, {type: "float"});
                    }
                    catch(e){
                        amount = 0;
                    }
                    if (!node.line.cashregister.journal.use_points) {
                        node.line.set_amount(amount);
                    } else {  // Check if we are receiving points or money
                        var repaint = event.which === 13 && true || false;
                        if (repaint) {
                            event.stopPropagation();
                        }
                        if (event.srcElement.id == 'money_input') {
                            node.line.set_amount_and_points(amount, repaint);
                        } else {
                            node.line.set_points_and_amount(amount, repaint);
                        }
                    }
                }
            };
            this.hotkey_handler = function(event){
                if(event.which === 13){
                    self.validate_order({invoice: true});
                }else if(event.which === 27){
                    self.back();
                }
            };
        },

        validate_order: function(options) {
            var self = this;
            options = options || {};
            var currentOrder = this.pos.get('selectedOrder');
            if (currentOrder.get_client().name === 'Consumidor Final') { 
                this.pos.pos_widget.screen_selector.show_popup('error',{ 
                    'message':_t('Error: Identificar al cliente'), 
                    'comment': _t('No se puede vender al cliente consumidor final'), 
                }); 
                return; 
            }
            var plines = currentOrder.get('paymentLines').models;
            var orderlines = currentOrder.get('orderLines').models;
	    var consignation = orderlines[0].consignation
	    for (var i = 0, len = orderlines.length; i < len; i++) {
	        if (orderlines[i].consignation != consignation) {
                    this.pos.pos_widget.screen_selector.show_popup('error',{
                        'message':_t('Error: Consignacion'),
                        'comment': _t('No se puede combinar productos sin consignacion, con productos que si poseen.'),
                    });
		    return;
		}	
	    }
            // Check if it is generic customer raise exception
//            var generic_customer = self.pos.config.default_partner_id[0];
//            var current_customer = currentOrder.get_client().id;
            if (currentOrder.get_client().name === 'Consumidor Final') {
                this.pos.pos_widget.screen_selector.show_popup('error',{
                    'message':_t('Error: Identificar al cliente'),
                    'comment': _t('No se puede vender al cliente consumidor final'),
                });
                return;
            }
            // Check done
            for (var i = 0; i < plines.length; i++) {
                if (plines[i].cashregister.journal.pos_payment_type == 'financial') {
                    if (plines[i].card_number == null || plines[i].card_number === "") {
                        this.pos.pos_widget.screen_selector.show_popup('error',{
                            'message':_t('Error: Card Number Required'),
                            'comment': _t('You need to provide the customer card number.'),
                        });
                        return;
                    }
                    try {
                        var q_qty = parseFloat(plines[i].quota_qty);
                        var q_amo = parseFloat(plines[i].quota_amount);
                        if (q_qty === 0) {
                            throw "Invalid quota qty: " + q_qty;
                        }
                        if (q_amo === 0) {
                            throw "Invalid quota amount: " + q_amo;
                        }
                    } catch (e) {
                        /* handle error */
                        this.pos.pos_widget.screen_selector.show_popup('error',{
                            'message':_t('Error'),
                            'comment': _t('Invalid quota_amount.'),
                        });
                        return;
                    }
                    if (plines[i].card_id == null || plines[i].card_id === "") {
                        this.pos.pos_widget.screen_selector.show_popup('error',{
                            'message':_t('Error: Card Required'),
                            'comment': _t('You need to provide the card issuer.'),
                        });
                        return;
                    }
                }
            }
            var tot_points_to_use = 0;
            var tot_credit_to_use = 0;
            var points_to_cash = 0;
            var client_points = parseFloat(currentOrder.attributes.client.ws_points || 0);
            var client_credit = parseFloat(currentOrder.attributes.client.ws_credit || 0);
            for (var i = 0; i < plines.length; i++) {
                if (plines[i].cashregister.journal.credipaz_card) {
                    tot_credit_to_use += plines[i].amount;
                }
                if (plines[i].cashregister.journal.use_points) {
                    tot_points_to_use += plines[i].points;
                    points_to_cash = Math.round(((plines[i].points / plines[i].cashregister.journal.points_rate)) * 100) / 100;
                    if (points_to_cash != plines[i].amount) {
                        this.pos.pos_widget.screen_selector.show_popup('error',{
                            'message':_t('Error: Points and Money does not match'),
                            'comment': _t('Maybe you have forgotten to recalculate the points or the amount.'),
                        });
                        return;
                    }
                }
            }
            if ((tot_points_to_use !== 0) && (tot_points_to_use > client_points)) {
                this.pos.pos_widget.screen_selector.show_popup('error',{
                    'message':_t('Error: Not enough points'),
                    'comment': _t('Trying to use more points than customer has available.'),
                });
                return;
            }
            if (parseFloat(tot_credit_to_use.toFixed(2)) > client_credit) {
                this.pos.pos_widget.screen_selector.show_popup('error',{
                    'message':_t('Error: Not enough credit'),
                    'comment': _t('Trying to use more credit than customer has available in credipaz card.'),
                });
                return;
            }
	    self.pos.get_order().consignation = consignation;
            return this._super(options);
        },

        render_paymentline: function(line){
            var self = this;
            var el_node = this._super(line);
            if (line.cashregister.journal.use_points) {
                var pts_in = el_node.querySelector('input#points_input');
                pts_in.addEventListener('keyup', this.line_change_handler);
                pts_in.addEventListener('click', this.line_click_handler);
            }
            if (line.cashregister.journal.pos_payment_type == 'financial') {
                var card_number_in = el_node.querySelector('input#card_number_input');
                card_number_in.addEventListener('keyup', this.line_card_handler);
                card_number_in.addEventListener('click', this.line_card_handler);
                var mp_input_sel = el_node.querySelector('select#mp_input_sel');
                mp_input_sel.addEventListener('keyup', this.line_mp_input_handler);
                mp_input_sel.addEventListener('click', this.line_mp_input_handler);
                var mp_qty_sel = el_node.querySelector('select#mp_qty_sel');
                mp_qty_sel.addEventListener('keyup', this.line_mp_qty_handler);
                mp_qty_sel.addEventListener('click', this.line_mp_qty_handler);
            }
            return el_node;
        },

        get_cash_amount: function () {
            var currentOrder = this.pos.get('selectedOrder');
            var paymentlines = currentOrder.get('paymentLines').models;

            var cash_payments = paymentlines.filter(function(p) {
                return p.cashregister.journal.pos_payment_type === 'cash';
            })

            var cash_amount = cash_payments.reduce(function(sum, payment) {
                return sum + payment.amount;
            }, 0)
             
            var any_cp_financial_payments = paymentlines.filter(function(p) {return p.cashregister.journal.credipaz_card == true}).length
            if (any_cp_financial_payments > 0) {
                var not_credipaz_financial_payments = paymentlines.filter(function(p) {
                    return p.cashregister.journal.pos_payment_type === 'financial' && p.cashregister.journal.credipaz_card === false;
                })
                var not_credipaz_financial_payments_amount = not_credipaz_financial_payments.reduce(function(vant, vact, iact, arr) { return vant + vact['amount'] }, 0)
                cash_amount += not_credipaz_financial_payments_amount;
            }

            var cash_amount_fixed = Math.round(cash_amount * 100) / 100;
            return cash_amount_fixed;
        },

        update_financing_summary: function () {
            var currentOrder = this.pos.get('selectedOrder');
            var quotas_amount = currentOrder.get_quotas_total();
            var financings = [];
            if (currentOrder.get('paymentLines').length != 0) {
                
                var financial_pl = currentOrder.get('paymentLines').filter(function(pl){
                    return pl.cashregister.journal.pos_payment_type === 'financial';   
                });
                financings = _.filter(financial_pl, function(el){
                    if (el.quota_amount != null || el.quota_qty != null) {
                        return el;
                    }
                });
            }
            // Build list of strings
            if (financings.length != 0) {
                var quotas_list = _.map(financings, function(x){
                    var quota_summary = '<span class="left-block">' +
                                        '<i class="fa fa-tag"/> Financiado<br/>'+x['name']+':' +
                                        '</span>' +
                                        '<span class="right-block payment-financing">'+x['quota_qty']+' x '+x['quota_amount']+'</span>';
                    return quota_summary;
                });
                this.$('.payment-financings').html(quotas_list.join(''));
            } else {  // no financed, blank field
                this.$('.payment-financings').html('');
            }
        },

        update_payment_summary: function () {
            var ret = this._super();
            this.update_financing_summary();
            return ret;
        },

    });
}
