/******************************************************************************
 *    Financing - Financing on Odoo
 *    Copyright (C) 2017 Eynes
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Affero General Public License as
 *    published by the Free Software Foundation, either version 3 of the
 *    License, or (at your option) any later version.
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Affero General Public License for more details.
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

function financing_widgets(instance, module) {
    // TODO: Here inherit instance.web.ListView.do_activate_record, and if view search of pos_order when clicking retrieve the pos.order that corresponds
    // to the clicked one

    var QWeb = instance.web.qweb,
    _t = instance.web._t;
    var round_di = instance.web.round_decimals;
    
    module.OrderListScreenWidget = module.OrderListScreenWidget.extend({
        show: function() {
            var self = this;
            this._super();
            this.$('.retrieve_order').off().click(function() {
                var inputVal = $(this.parentElement).find('.searchbox > input').val();
                self.do_retrieve_order(inputVal);
            });
        },
        do_retrieve_order: function(inputVal) {
            var self = this;
            
            var models_to_fetch = _.filter(self.pos.models, function(el){
                return (el.model == 'pos.order')||(el.model == 'pos.order.line')||(el.model == 'account.bank.statement.line');
            });
            
            loaded = self.pos.do_lookup_retrieve_order(models_to_fetch, inputVal);
            loaded.fail(function (error){
                // if(error.code === 200 ){    // Business Logic Error, not a connection problem
                //     self.pos_widget.screen_selector.show_popup('error-traceback',{
                //         message: error.data.message,
                //         comment: error.data.debug
                //     });
                // } else {
                //     self.pos_widget.screen_selector.show_popup('error-traceback',{
                //         message: _t('Unable to fetch orders'),
                //         comment: _t('Maybe there is a connection problem')
                //     });
                // }
                console.error('Failed to retrieve orders:', error);
                event.preventDefault();
            });
            loaded.done(function(){
                this.display_order(inputVal);
            }.bind(this, inputVal));

        },
    });

    module.PaymentScreenWidget.include({
        validate_order: function(options) {
            var self = this;
            options = options || {};

            var currentOrder = this.pos.get('selectedOrder');

            if( !currentOrder.get_client()){
                self.pos_widget.screen_selector.show_popup('error',{
                    message: _t('An anonymous order cannot be confirmed'),
                    comment: _t('Please select a client for this order. This can be done by clicking the order tab')
                });
                return;
            }
            var orderLines = currentOrder.get('orderLines').models;
            // Search all the combos being sold, then get all the products of that combos
            // Finally check that all the components are present. If not raise exception.
            var comboLines = _.filter(orderLines, function(x){return x.product.is_combo === true;});
            var productCombos = _.map(comboLines, function(x){return x.product;});
            var expectedComponents = [];
            for (var i = 0, len = productCombos.length; i < len; i++) {
                expectedComponents = expectedComponents.concat(productCombos[i].component_ids);
            }
            for (var i = 0, len = orderLines.length; i < len; i++) {
                var pid = orderLines[i].product.id;
                var idx = expectedComponents.indexOf(pid);
                if (idx > -1) {
                    expectedComponents.splice(idx, 1);
                }
            }
            if (expectedComponents.length !== 0) {
                self.pos_widget.screen_selector.show_popup('error',{
                    message: _t('Combo Error'),
                    comment: _t('Combo & Components quantity mismatch. Please verify the quantities of combo you are trying to sell.')
                });
                return;
            }
            // else
            return this._super(options);
        }
    });

    module.ClientListScreenWidget.include({
        show: function(){
            var self = this;
            currOrder = self.pos.get('selectedOrder');
            if (!currOrder.attributes.client || 'Consumidor Final' === currOrder.attributes.client.name) {
                return self._super();
            }
            self.pos_widget.screen_selector.show_popup('error',{
                message: _t('Cannot change a customer previously defined'),
                comment: _t('Please delete the current order to change the customer')
            });
            return;
        },
    });

    module.OrderButtonWidget = module.OrderButtonWidget.extend({
        selectOrder: function (event) {
            // this function is called when creating or changing between orders 
            this._super(event);
            var partner = this.order.get_client() ? this.order.get_client() : false;
        },
        renderElement: function () {
            this._super();
            var order = posmodel.get_order();
            if(order && order.get_client()) {
                order.update_partner_ui_tooltip(order);
            }
        }
    });
}
