# -*- coding: utf-8 -*-
from openerp.exceptions import except_orm, ValidationError
from openerp.tests.common import TransactionCase
from openerp import fields
from datetime import datetime, timedelta, date
from tempfile import TemporaryFile
import logging
import pytest

class TestFinancingExceptions(TransactionCase):
    def tearDown(self):
        super(TestFinancingExceptions, self).tearDown()

    def setUp(self):
        super(TestFinancingExceptions, self).setUp()
        self.cr.execute('delete from financing_config')
        translate_logger = logging.getLogger('openerp.tools.translate')
        translate_logger.setLevel('INFO')  # mute translations
        self.fc_model = self.env['financing.config']
        self.journal = self.browse_ref('financing.card_journal01')
        self.pl0 = self.browse_ref('product.list0')
        self.pl1 = self.browse_ref('financing.list1')
        self.card01 = self.browse_ref('financing.card01')
        self.card02 = self.browse_ref('financing.card02')
        self.pos_config_main = self.browse_ref('point_of_sale.pos_config_main')
        self.pos_config_main2 = self.ref('financing.pos_config_main2')

    def test_different_rates_for_one_quota_coeff_collision(self):
        """
        Different monthly rates for same quota should raise exception
        """
        self.fcgeneric_ass = self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'customer_type': 'associated',
            'quota_qty': 1,
            'monthly_rate': 0.4,
        })
        self.fcgeneric_nonass = self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'customer_type': 'associated',
            'quota_qty': 1,
            'monthly_rate': 0.5,
        })
        with pytest.raises(ValidationError) as e:
            coeff = self.fc_model.lookup_financing(
                product_ids=[self.ref('product.product_product_5')],
                journal_id=self.journal.id,
                customer_type='associated',
                pricelist_id=self.pl0.id,
                pos_config_id=self.pos_config_main.id,
                card_id=self.card01.id,
                quota_qty=1,
            )
        assert 'More than one rate available' in str(e.value)

    def test_getting_coeff_for_two_products_but_one_with_disabled_financing(self):
        """
        If one of both financings is disabled, an exception should be raised
        """
        fc01 = self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'customer_type': 'associated',
            'quota_qty': 1,
            'monthly_rate': 0.4,
            'product_ids': [(6, 0, [self.ref('product.product_product_5')])],
        })
        fc02 = self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'customer_type': 'associated',
            'quota_qty': 1,
            'monthly_rate': 0.4,
            'product_ids': [(6, 0, [self.ref('product.product_product_4')])],
        })
        coeff = self.fc_model.lookup_financing(
            product_ids=[self.ref('product.product_product_4'), self.ref('product.product_product_5')],
            journal_id=self.journal.id,
            customer_type='associated',
            pricelist_id=self.pl0.id,
            pos_config_id=self.pos_config_main.id,
            card_id=self.card02.id,
            quota_qty=1,
        )
        assert coeff == 1.4
        fc01.disabled = True
        with pytest.raises(ValidationError) as e:
            coeff = self.fc_model.lookup_financing(
                product_ids=[self.ref('product.product_product_4'), self.ref('product.product_product_5')],
                journal_id=self.journal.id,
                customer_type='associated',
                pricelist_id=self.pl0.id,
                pos_config_id=self.pos_config_main.id,
                card_id=self.card01.id,
                quota_qty=1,
            )
        assert 'The financing has been disabled' in str(e.value)
