# -*- coding: utf-8 -*-
from openerp import models, fields, api, exceptions, _, SUPERUSER_ID
from openerp.addons import decimal_precision as dp
from lxml import etree
import logging
from openerp.tools import float_compare, float_round

_logger = logging.getLogger(__name__)


class PosMakePayment(models.TransientModel):
    _inherit = 'pos.make.payment'

    fc_id = fields.Many2one('financing.config', string='Financing')
    order_id = fields.Many2one('pos.order', string='Order')
    coeff = fields.Float(string='Coefficient',
                         digits=dp.get_precision('Financing Config.'),
                         related='fc_id.coeff')
    card_id = fields.Many2one('payment.card', string='Card')
    pos_payment_type = fields.Selection(related='journal_id.pos_payment_type', readonly=1)
    use_points = fields.Boolean(related='journal_id.use_points', readonly=1)
    financing_description = fields.Char('Financing Description', compute='compute_amount')
    quota_amount = fields.Float('Quota Amount', compute='compute_amount')
    quota_qty = fields.Integer(related='fc_id.quota_qty', readonly=1)
    card_number = fields.Char('Card Number')
    quantity_points = fields.Float('Qty. Points')

    @api.constrains('amount')
    def _check_amount(self):
        """ Ensure the partner is the same or empty on all lines and a reconcile mark.
            We allow that only for opening/closing period"""
        total_order = abs(self.order_id.amount_total)
        total_paid = abs(self.order_id.amount_paid)
        todo = abs(self.amount)
        remainder = total_order - total_paid
        if todo > remainder:
            raise exceptions.ValidationError(_('Invalid amount %s > %s') % (todo, remainder))

    @api.onchange('quantity_points')
    def onchange_qty_points(self):
        if self.journal_id.use_points:
            order = self.env['pos.order'].browse(self._context['active_ids'])[0]
            currency = order.pricelist_id.currency_id
            amount = self.quantity_points / self.journal_id.points_rate
            self.amount = currency.round(amount)

    @api.model
    def default_get(self, fields):
        res = super(PosMakePayment, self).default_get(fields)
        ctx = self._context
        if ctx.get('active_model') == 'pos.order':
            res['order_id'] = ctx.get('active_id', False)
        return res

    @api.depends('amount', 'fc_id')
    def compute_amount(self):
        if self.fc_id:
            order = self.env['pos.order'].browse(self._context['active_ids'])[0]
            currency = order.pricelist_id.currency_id
            total_financed = self.fc_id.coeff * self.amount
            quota_amount = total_financed / self.fc_id.quota_qty
            self.quota_amount = currency.round(quota_amount)
            self.financing_description = '%s x $%s' % (self.fc_id.quota_qty, abs(self.quota_amount))
        else:
            self.quota_amount = 0
            self.financing_description = ''

    @api.onchange('journal_id')
    def onchange_journal_id(self):
        card_id_domain = [('id', 'in', self.journal_id.card_ids.ids)]
        res = {
            'domain': {
                'card_id': card_id_domain,
            },
        }
        self.card_id = False
        return res

    @api.onchange('card_id')
    def onchange_card_id(self):
        """ Filter the selection of quotas to what is available for that card """
        if self.card_id:
            order = self.env['pos.order'].browse(self._context['active_ids'])[0]
            fc = self.env['financing.config'].with_context({'return_fcid': True})
            product_ids = order.lines.mapped('product_id').ids
            fcr = fc.perform_search(
                product_ids,
                self.journal_id.id,
                order.partner_id.id,
                order.pricelist_id.id,
                order.session_id.config_id.id,
            )
            financings_for_card = fcr.get(self.card_id.id, False)
            if financings_for_card:
                if isinstance(financings_for_card, tuple):
                    avail_fc = [financings_for_card[1]]
                else:
                    avail_fc = [v[1] for k, v in financings_for_card.items()]
                self.fc_id = False
                res = {
                    'domain': {
                        'fc_id': [('id', 'in', avail_fc)]
                    }
                }
            else:
                self.fc_id = False
                res = {
                    'domain': {
                        'fc_id': [('id', '=', False)]
                    }
                }
        else:
            self.fc_id = False
            res = {
                'domain': {
                    'fc_id': [('id', '=', False)]
                }
            }
        return res

    @api.model
    def fields_view_get(self, view_id=None, view_type=False, toolbar=False, submenu=False):
        """ Filter the journal_id to whatever journals the pos.session has """
        context = self._context
        res = super(PosMakePayment, self).fields_view_get(view_id=view_id, view_type=view_type, toolbar=toolbar, submenu=submenu)
        if view_type == 'form':
            if context.get('active_model') == 'pos.order' and context.get('active_ids'):
                order = self.env['pos.order'].browse(context['active_ids'])[0]
                journal_ids = order.session_id.journal_ids.ids
                doc = etree.XML(res['arch'])
                domain = "[('id', 'in', %s)]" % journal_ids
                for node in doc.xpath("//field[@name='journal_id']"):
                    node.set('domain', domain)
                res['arch'] = etree.tostring(doc)
        return res
